package jcg.zheng.demo.web.security;

import java.io.IOException;
import java.util.List;

import javax.ws.rs.NotAuthorizedException;
import javax.ws.rs.container.ContainerRequestContext;
import javax.ws.rs.container.ContainerRequestFilter;
import javax.ws.rs.container.ResourceInfo;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MultivaluedMap;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.core.annotation.AnnotationUtils;
import org.springframework.util.CollectionUtils;

import jcg.zheng.demo.customannotation.RequiresPermissions;
import jcg.zheng.demo.web.data.User;
import jcg.zheng.demo.web.service.UserService;

public class RequiresPermissionsFilter implements ContainerRequestFilter {

	private static final String SUPER_USER = "MZheng";

	@Context
	private ApplicationContext applicationContext;

	@Context
	private ResourceInfo resourceInfo;

	@Autowired
	private UserService userSrv;

	@Override
	public void filter(ContainerRequestContext requestContext) throws IOException {
		RequiresPermissions annotation = AnnotationUtils.findAnnotation(resourceInfo.getResourceMethod(),
				RequiresPermissions.class);
		if (annotation != null) {
			MultivaluedMap<String, String> headers = requestContext.getHeaders();
			processPermission(headers, annotation);
		}

	}

	private void processPermission(MultivaluedMap<String, String> headers, RequiresPermissions permission) {
		String permissionValue = permission.value();
		String permissionType = permission.type();
		if ("role".equalsIgnoreCase(permissionType)) {
			// need to check the header user id's role match to the permission role
			List<String> requestUserId = headers.get("requestUserId");
			if (CollectionUtils.isEmpty(requestUserId)) {
				throw new NotAuthorizedException("Missing security header");
			}

			if (!requestUserId.get(0).equalsIgnoreCase(SUPER_USER)) {
				Integer requestUserNum = Integer.valueOf(requestUserId.get(0));
				User requestUser = userSrv.getUser(requestUserNum);
				if (requestUser == null) {
					throw new NotAuthorizedException("Invalid requestUserId");
				}
				Integer userRoleInt = Integer.valueOf(requestUser.getRole());
				Integer permissionRoleInt = Integer.valueOf(permissionValue);
				if (userRoleInt < permissionRoleInt) {
					throw new NotAuthorizedException(
							"Not Authorized for the method, request user must have a role=" + permissionValue);
				}
			}
		}
	}
}
