package jcg.zheng.demo.web.service;

import java.util.Collections;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import jcg.zheng.demo.web.data.Person;
import jcg.zheng.demo.web.data.User;
import jcg.zheng.demo.web.repository.PersonRepository;

@Service
@Transactional(isolation = Isolation.READ_UNCOMMITTED)
public class UserService {

	@Autowired
	private PersonRepository personRepo;

	public User getUser(Integer userId) {
		Optional<Person> p = personRepo.findById(userId);
		if (p.isEmpty()) {
			return null;
		}

		return new User(p.get());
	}

	public List<User> getUsers() {
		List<Person> persons = personRepo.findAll();

		if (CollectionUtils.isEmpty(persons)) {
			return Collections.emptyList();
		}
		return persons.stream().map(p -> new User(p)).collect(Collectors.toList());
	}

	@Transactional(isolation = Isolation.READ_COMMITTED)
	public User saveUser(User user) {
		Person p = new Person(user);
		Person saved = personRepo.save(p);
		return new User(saved);
	}

}
